defmodule Archeometer.Analysis.Treemap.Data do
  @moduledoc false

  import Archeometer.Query
  alias Archeometer.Repo
  alias Archeometer.Schema.Module

  def get_data(:size, app, namespace, db_name, skip_tests) do
    get_modules_and_lcount(app, namespace, db_name, skip_tests)
  end

  defp get_modules_and_lcount(app, ns, db_name, skip_tests) do
    modules = get_modules(app, ns, db_name)

    modules =
      if skip_tests do
        Enum.filter(modules, &(is_test?(&1) == false))
      else
        modules
      end

    Enum.map(modules, fn [id, name, num_lines, _path] -> {id, name, num_lines} end)
  end

  defp get_modules(:none, "*", db_name) do
    %{rows: modules} =
      Repo.all(
        from(m in Module,
          select: [
            id: m.id,
            name: m.name,
            num_lines: m.num_lines,
            path: m.path
          ]
        ),
        [],
        db_name
      )

    modules
  end

  defp get_modules(:none, ns, db_name) do
    %{rows: modules} =
      Repo.all(
        from(m in Module,
          select: [
            id: m.id,
            name: m.name,
            num_lines: m.num_lines,
            path: m.path
          ],
          where: m.name == ^ns or like(m.name, ^(ns <> ".%"))
        ),
        [],
        db_name
      )

    modules
  end

  defp get_modules(app, "*", db_name) do
    app_str = to_string(app)

    %{rows: modules} =
      Repo.all(
        from(m in Module,
          select: [
            id: m.id,
            name: m.name,
            num_lines: m.num_lines,
            path: m.path
          ],
          where: m.application.name == ^app_str
        ),
        [],
        db_name
      )

    modules
  end

  defp get_modules(app, ns, db_name) do
    app_str = to_string(app)

    %{rows: modules} =
      Repo.all(
        from(m in Module,
          select: [
            id: m.id,
            name: m.name,
            num_lines: m.num_lines,
            path: m.path
          ],
          where: m.application.name == ^app_str and (m.name == ^ns or like(m.name, ^(ns <> ".%")))
        ),
        [],
        db_name
      )

    modules
  end

  defp is_test?([_id, _name, _num_kines, path]) do
    if Mix.Project.umbrella?() or is_archeometer_test?() do
      Regex.match?(~r{apps/.+/test/}, path)
    else
      String.starts_with?(path, "test/")
    end
  end

  defp is_archeometer_test?() do
    Mix.env() == :test and Keyword.get(Mix.Project.config(), :app) == :archeometer
  end
end
