defmodule Archeometer.Util.PathHelper do
  @moduledoc false

  def template_path(template) do
    Path.expand(template, template_path())
  end

  def template_path() do
    case Mix.Project.get() do
      # When archeometer is the current project
      Archeometer.MixProject ->
        "./lib/templates/"

      _ ->
        case archeometer_config() do
          # When archeometer is a local dep
          {:local, path} ->
            Path.expand("lib/templates", path)

          _ ->
            # When archeometer is a remote dep (downloaded to 'deps_path')
            Path.expand("archeometer/lib/templates/", Mix.Project.deps_path())
        end
    end
  end

  defp archeometer_config do
    Mix.Project.config()
    |> Keyword.get(:deps)
    |> Enum.find(:not_found, fn dep -> elem(dep, 0) == :archeometer end)
    |> process_archeometer_config()
  end

  defp process_archeometer_config({:archeometer, opts}) do
    if Keyword.keyword?(opts) do
      case Keyword.fetch(opts, :path) do
        {:ok, path} -> {:local, path}
        :error -> :remote
      end
    else
      :remote
    end
  end
end
